/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.LinearGradientPaint;
import java.awt.RadialGradientPaint;
import java.awt.TexturePaint;
import java.awt.MultipleGradientPaint.CycleMethod;
import java.awt.geom.Rectangle2D;



/**
 * This class contains static methods for setting various rendering styles used
 * for drawing entities in gfx. tests.
 * 
 * @author Pavel Tisnovsky
 */
public class CommonRenderingStyles
{
    /**
     * Stroke width used for drawing "thick" curves.
     */
    private static final int STROKE_WIDTH_THICK = 10;

    /**
     * Stroke width used for drawing extra "thick" curves.
     */
    private static final int STROKE_WIDTH_EXTRA_THICK = 30;

    /**
     * Default dash pattern.
     */
    private static final float[] DEFAULT_DASH_PATTERN = {20.0f};

    /**
     * Default dash pattern.
     */
    private static final float[] DEFAULT_DASH_PATTERN_FOR_EXTRA_THICK_PATHS = {40.0f};

    /**
     * Default stroke color.
     */
    private static final Color DEFAULT_STROKE_COLOR = Color.BLACK;

    /**
     * Default fill color.
     */
    private static final Color DEFAULT_FILL_COLOR = Color.GREEN.darker();

    /**
     * Default gradient paint first color.
     */
    private static final Color DEFAULT_GRADIENT_PAINT_FIRST_COLOR = Color.BLUE;

    /**
     * Default gradient paint second color.
     */
    private static final Color DEFAULT_GRADIENT_PAINT_SECOND_COLOR = Color.YELLOW;

    /**
     * Default linear gradient paint first color.
     */
    private static final Color DEFAULT_LINEAR_GRADIENT_PAINT_FIRST_COLOR = Color.RED;

    /**
     * Default linear gradient paint second color.
     */
    private static final Color DEFAULT_LINEAR_GRADIENT_PAINT_SECOND_COLOR = Color.WHITE;

    /**
     * Default linear gradient paint third color.
     */
    private static final Color DEFAULT_LINEAR_GRADIENT_PAINT_THIRD_COLOR = Color.BLUE;

    /**
     * Default radial gradient paint first color.
     */
    private static final Color DEFAULT_RADIAL_GRADIENT_PAINT_FIRST_COLOR = Color.RED;

    /**
     * Default radial gradient paint second color.
     */
    private static final Color DEFAULT_RADIAL_GRADIENT_PAINT_SECOND_COLOR = Color.WHITE;

    /**
     * Default radial gradient paint third color.
     */
    private static final Color DEFAULT_RADIAL_GRADIENT_PAINT_THIRD_COLOR = Color.BLUE;

    /**
     * Set default stroke color.
     * 
     * @param graphics
     *            graphics context for image
     */
    public static void setStrokeColor(Graphics2D graphics)
    {
        graphics.setColor(DEFAULT_STROKE_COLOR);
    }

    /**
     * Set default fill color.
     * 
     * @param graphics
     *            graphics context for image
     */
    public static void setFillColor(Graphics2D graphics)
    {
        graphics.setPaint(DEFAULT_FILL_COLOR);
    }

    /**
     * Make sure the transparency is within given bounds 0.0-1.0
     * 
     * @param transparency
     *            color transparency
     * @return transparency in bounds 0.0-1.0
     */
    private static float transparencyToBounds(float transparency)
    {
        return transparency < 0.f ? 0.f : transparency > 1.f ? 1.f : transparency;
    }

    /**
     * Set transparent color.
     * 
     * @param graphics
     *            graphics context for image
     * @param color
     *            color without transparency
     * @param transparency
     *            color transparency represented in percents (0..100)
     */
    public static void setTransparentFillColor(Graphics2D graphics, Color color, int transparency)
    {
        float[] rgb = color.getRGBColorComponents(null);
        float value = transparencyToBounds(transparency / 100.0f);
        graphics.setPaint(new Color(rgb[0], rgb[1], rgb[2], value));
    }

    /**
     * Set transparent black color.
     * 
     * @param graphics
     *            graphics context for image
     * @param transparency
     *            color transparency represented in percents (0..100)
     */
    public static void setTransparentFillBlackColor(Graphics2D graphics, int transparency)
    {
        float value = transparencyToBounds(transparency / 100.0f);
        graphics.setPaint(new Color(0.0f, 0.0f, 0.0f, value));
    }

    /**
     * Set transparent red color.
     * 
     * @param graphics
     *            graphics context for image
     * @param transparency
     *            color transparency represented in percents (0..100)
     */
    public static void setTransparentFillRedColor(Graphics2D graphics, int transparency)
    {
        float value = transparencyToBounds(transparency / 100.0f);
        graphics.setPaint(new Color(1.0f, 0.0f, 0.0f, value));
    }

    /**
     * Set transparent green color.
     * 
     * @param graphics
     *            graphics context for image
     * @param transparency
     *            color transparency represented in percents (0..100)
     */
    public static void setTransparentFillGreenColor(Graphics2D graphics, int transparency)
    {
        float value = transparencyToBounds(transparency / 100.0f);
        graphics.setPaint(new Color(0.0f, 1.0f, 0.0f, value));
    }

    /**
     * Set transparent blue color.
     * 
     * @param graphics
     *            graphics context for image
     * @param transparency
     *            color transparency represented in percents (0..100)
     */
    public static void setTransparentFillBlueColor(Graphics2D graphics, int transparency)
    {
        float value = transparencyToBounds(transparency / 100.0f);
        graphics.setPaint(new Color(0.0f, 0.0f, 1.0f, value));
    }

    /**
     * Set transparent magenta color.
     * 
     * @param graphics
     *            graphics context for image
     * @param transparency
     *            color transparency represented in percents (0..100)
     */
    public static void setTransparentFillMagentaColor(Graphics2D graphics, int transparency)
    {
        float value = transparencyToBounds(transparency / 100.0f);
        graphics.setPaint(new Color(1.0f, 0.0f, 1.0f, value));
    }

    /**
     * Set transparent cyan color.
     * 
     * @param graphics
     *            graphics context for image
     * @param transparency
     *            color transparency represented in percents (0..100)
     */
    public static void setTransparentFillCyanColor(Graphics2D graphics, int transparency)
    {
        float value = transparencyToBounds(transparency / 100.0f);
        graphics.setPaint(new Color(0.0f, 1.0f, 1.0f, value));
    }

    /**
     * Set transparent yellow color.
     * 
     * @param graphics
     *            graphics context for image
     * @param transparency
     *            color transparency represented in percents (0..100)
     */
    public static void setTransparentFillYellowColor(Graphics2D graphics, int transparency)
    {
        float value = transparencyToBounds(transparency / 100.0f);
        graphics.setPaint(new Color(1.0f, 1.0f, 0.0f, value));
    }

    /**
     * Set transparent white color.
     * 
     * @param graphics
     *            graphics context for image
     * @param transparency
     *            color transparency represented in percents (0..100)
     */
    public static void setTransparentFillWhiteColor(Graphics2D graphics, int transparency)
    {
        float value = transparencyToBounds(transparency / 100.0f);
        graphics.setPaint(new Color(1.0f, 1.0f, 1.0f, value));
    }

    /**
     * Set horizontal gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setHorizontalGradientFill(TestImage image, Graphics2D graphics)
    {
        setHorizontalGradientFill(image, graphics, DEFAULT_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_GRADIENT_PAINT_SECOND_COLOR);
    }

    /**
     * Set horizontal gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param offset
     *            offset between shape border and edge of horizontal gradient,
     *            positive offset: gradient is narrower, negative offset:
     *            gradient is broader
     */
    public static void setHorizontalGradientFill(TestImage image, Graphics2D graphics, int offset)
    {
        setHorizontalGradientFill(image, graphics, DEFAULT_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_GRADIENT_PAINT_SECOND_COLOR, offset);
    }

    /**
     * Set horizontal gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     */
    public static void setHorizontalGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2)
    {
        setHorizontalGradientFill(image, graphics, color1, color2, 0);
    }

    /**
     * Set horizontal gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     * @param offset
     *            offset between shape border and edge of horizontal gradient
     */
    public static void setHorizontalGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, int offset)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);
        // set smaller radius to better see the gradient
        radius = radius >> 1;

        GradientPaint gradientPaint = new GradientPaint(xc - radius + offset, yc, color1, xc + radius - offset, yc, color2);
        graphics.setPaint(gradientPaint);
    }

    /**
     * Set vertical gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setVerticalGradientFill(TestImage image, Graphics2D graphics)
    {
        setVerticalGradientFill(image, graphics, DEFAULT_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_GRADIENT_PAINT_SECOND_COLOR);
    }

    /**
     * Set vertical gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param offset
     *            offset between shape border and edge of horizontal gradient,
     *            positive offset: gradient is narrower, negative offset:
     *            gradient is broader
     */
    public static void setVerticalGradientFill(TestImage image, Graphics2D graphics, int offset)
    {
        setVerticalGradientFill(image, graphics, DEFAULT_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_GRADIENT_PAINT_SECOND_COLOR, offset);
    }

    /**
     * Set vertical gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     */
    public static void setVerticalGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2)
    {
        setVerticalGradientFill(image, graphics, color1, color2, 0);
    }

    /**
     * Set vertical gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     * @param offset
     *            offset between shape border and edge of horizontal gradient
     */
    public static void setVerticalGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, int offset)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);
        // set smaller radius to better see the gradient
        radius = radius >> 1;

        GradientPaint gradientPaint = new GradientPaint(xc, yc - radius + offset, color1, xc, yc + radius - offset, color2);
        graphics.setPaint(gradientPaint);
    }

    /**
     * Set diagonal gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setDiagonalGradientFill(TestImage image, Graphics2D graphics)
    {
        setDiagonalGradientFill(image, graphics, DEFAULT_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_GRADIENT_PAINT_SECOND_COLOR);
    }

    /**
     * Set diagonal gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param offset
     *            offset between shape border and edge of horizontal gradient,
     *            positive offset: gradient is narrower, negative offset:
     *            gradient is broader
     */
    public static void setDiagonalGradientFill(TestImage image, Graphics2D graphics, int offset)
    {
        setDiagonalGradientFill(image, graphics, DEFAULT_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_GRADIENT_PAINT_SECOND_COLOR, offset);
    }

    /**
     * Set diagonal gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     */
    public static void setDiagonalGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2)
    {
        setDiagonalGradientFill(image, graphics, color1, color2, 0);
    }

    /**
     * Set diagonal gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     * @param offset
     *            offset between shape border and edge of horizontal gradient
     */
    public static void setDiagonalGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, int offset)
    {
        // sin 45 = cos 45 = 0.707f
        final double SIN_COS_45 = Math.cos(Math.PI / 4);

        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);
        // set smaller radius to better see color gradient
        radius = radius >> 1;
        int r2 = (int) (radius * SIN_COS_45);
        int delta = (int) (offset * SIN_COS_45);

        // compute end points used for diagonal fill
        int x1 = xc - r2 + delta;
        int y1 = yc - r2 + delta;
        int x2 = xc + r2 - delta;
        int y2 = yc + r2 - delta;

        GradientPaint gradientPaint = new GradientPaint(x1, y1, color1, x2, y2, color2);
        graphics.setPaint(gradientPaint);
    }

    /**
     * Set horizontal linear gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setHorizontalLinearGradientFill(TestImage image, Graphics2D graphics)
    {
        setHorizontalLinearGradientFill(image, graphics, DEFAULT_LINEAR_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_SECOND_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_THIRD_COLOR);
    }

    /**
     * Set horizontal linear gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param offset
     *            offset between shape border and edge of horizontal gradient,
     *            positive offset: gradient is narrower, negative offset:
     *            gradient is broader
     */
    public static void setHorizontalLinearGradientFill(TestImage image, Graphics2D graphics, int offset)
    {
        setHorizontalLinearGradientFill(image, graphics, DEFAULT_LINEAR_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_SECOND_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_THIRD_COLOR, offset);
    }

    /**
     * Set horizontal linear gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     * @param color3
     *            color set for the third edge of gradient fill
     */
    public static void setHorizontalLinearGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, Color color3)
    {
        setHorizontalLinearGradientFill(image, graphics, color1, color2, color3, 0);
    }

    /**
     * Set horizontal linear gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     * @param color3
     *            color set for the third edge of gradient fill
     * @param offset
     *            offset between shape border and edge of horizontal gradient
     */
    public static void setHorizontalLinearGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, Color color3, int offset)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);

        // numbers ranging from 0.0 to 1.0 specifying the distribution of colors along the gradient
        float[] fractions = { 0.0f, 0.5f, 1.0f };

        // array of colors corresponding to each fractional value
        Color[] colors = { color1, color2, color3 };

        // create linear gradient object
        LinearGradientPaint linearGradientPaint = new LinearGradientPaint(xc - radius + offset, yc, xc + radius - offset, yc, fractions, colors);

        // set fill style
        graphics.setPaint(linearGradientPaint);
    }

    /**
     * Set horizontal linear gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param fractions
     *            numbers ranging from 0.0 to 1.0 specifying the distribution of
     *            colors along the gradient
     * @param colors
     *            array of colors corresponding to each fractional value
     */
    public static void setHorizontalLinearGradientFill(TestImage image, Graphics2D graphics, float[] fractions, Color[] colors)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);

        // create linear gradient object
        LinearGradientPaint linearGradientPaint = new LinearGradientPaint(xc - radius, yc, xc + radius, yc, fractions, colors);

        // set fill style
        graphics.setPaint(linearGradientPaint);
    }

    /**
     * Set vertical linear gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setVerticalLinearGradientFill(TestImage image, Graphics2D graphics)
    {
        setVerticalLinearGradientFill(image, graphics, DEFAULT_LINEAR_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_SECOND_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_THIRD_COLOR);
    }

    /**
     * Set vertical linear gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param offset
     *            offset between shape border and edge of horizontal gradient,
     *            positive offset: gradient is narrower, negative offset:
     *            gradient is broader
     */
    public static void setVerticalLinearGradientFill(TestImage image, Graphics2D graphics, int offset)
    {
        setVerticalLinearGradientFill(image, graphics, DEFAULT_LINEAR_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_SECOND_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_THIRD_COLOR, offset);
    }

    /**
     * Set vertical linear gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     * @param color3
     *            color set for the third edge of gradient fill
     */
    public static void setVerticalLinearGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, Color color3)
    {
        setVerticalLinearGradientFill(image, graphics, color1, color2, color3, 0);
    }

    /**
     * Set vertical linear gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     * @param color3
     *            color set for the third edge of gradient fill
     * @param offset
     *            offset between shape border and edge of horizontal gradient
     */
    public static void setVerticalLinearGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, Color color3, int offset)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);

        // numbers ranging from 0.0 to 1.0 specifying the distribution of colors along the gradient
        float[] fractions = { 0.0f, 0.5f, 1.0f };

        // array of colors corresponding to each fractional value
        Color[] colors = { color1, color2, color3 };

        // create linear gradient object
        LinearGradientPaint linearGradientPaint = new LinearGradientPaint(xc, yc - radius + offset, xc, yc + radius - offset, fractions, colors);

        // set fill style
        graphics.setPaint(linearGradientPaint);
    }

    /**
     * Set vertical linear gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param fractions
     *            numbers ranging from 0.0 to 1.0 specifying the distribution of
     *            colors along the gradient
     * @param colors
     *            array of colors corresponding to each fractional value
     */
    public static void setVerticalLinearGradientFill(TestImage image, Graphics2D graphics, float[] fractions, Color[] colors)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);

        // create linear gradient object
        LinearGradientPaint linearGradientPaint = new LinearGradientPaint(xc, yc - radius, xc, yc + radius, fractions, colors);

        // set fill style
        graphics.setPaint(linearGradientPaint);
    }

    /**
     * Set diagonal linear gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setDiagonalLinearGradientFill(TestImage image, Graphics2D graphics)
    {
        setDiagonalLinearGradientFill(image, graphics, DEFAULT_LINEAR_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_SECOND_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_THIRD_COLOR);
    }

    /**
     * Set diagonal linear gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param offset
     *            offset between shape border and edge of horizontal gradient,
     *            positive offset: gradient is narrower, negative offset:
     *            gradient is broader
     */
    public static void setDiagonalLinearGradientFill(TestImage image, Graphics2D graphics, int offset)
    {
        setDiagonalLinearGradientFill(image, graphics, DEFAULT_LINEAR_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_SECOND_COLOR, DEFAULT_LINEAR_GRADIENT_PAINT_THIRD_COLOR, offset);
    }

    /**
     * Set diagonal linear gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     * @param color3
     *            color set for the third edge of gradient fill
     */
    public static void setDiagonalLinearGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, Color color3)
    {
        setDiagonalLinearGradientFill(image, graphics, color1, color2, color3, 0);
    }

    /**
     * Set diagonal linear gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color set for the first edge of gradient fill
     * @param color2
     *            color set for the second edge of gradient fill
     * @param color3
     *            color set for the third edge of gradient fill
     * @param offset
     *            offset between shape border and edge of horizontal gradient
     */
    public static void setDiagonalLinearGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, Color color3, int offset)
    {
        // sin 45 = cos 45 = 0.707f
        final double SIN_COS_45 = Math.cos(Math.PI / 4);

        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);
        int r2 = (int)(radius * SIN_COS_45 );
        int delta = (int) (offset * SIN_COS_45);

        // compute end points used for diagonal fill
        int x1 = xc - r2 + delta;
        int y1 = yc - r2 + delta;
        int x2 = xc + r2 - delta;
        int y2 = yc + r2 - delta;

        // numbers ranging from 0.0 to 1.0 specifying the distribution of colors along the gradient
        float[] fractions = { 0.0f, 0.5f, 1.0f };

        // array of colors corresponding to each fractional value
        Color[] colors = { color1, color2, color3 };

        // create linear gradient object
        LinearGradientPaint linearGradientPaint = new LinearGradientPaint(x1, y1, x2, y2, fractions, colors);

        // set fill style
        graphics.setPaint(linearGradientPaint);
    }

    /**
     * Set diagonal linear gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param fractions
     *            numbers ranging from 0.0 to 1.0 specifying the distribution of
     *            colors along the gradient
     * @param colors
     *            array of colors corresponding to each fractional value
     */
    public static void setDiagonalLinearGradientFill(TestImage image, Graphics2D graphics, float[] fractions, Color[] colors)
    {
        // sin 45 = cos 45 = 0.707f
        final double SIN_COS_45 = Math.cos(Math.PI / 4);

        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);
        int r2 = (int)(radius * SIN_COS_45 );

        // compute end points used for diagonal fill
        int x1 = xc - r2;
        int y1 = yc - r2;
        int x2 = xc + r2;
        int y2 = yc + r2;

        // create linear gradient object
        LinearGradientPaint linearGradientPaint = new LinearGradientPaint(x1, y1, x2, y2, fractions, colors);

        // set fill style
        graphics.setPaint(linearGradientPaint);
    }

    /**
     * Set radial gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param cycleMethod
     *            the method to use when painting outside gradient bounds
     * @param radiusScale
     *            scale used while calculating radius
     */
    public static void setRadialGradientFill(TestImage image, Graphics2D graphics, CycleMethod cycleMethod, float radiusScale)
    {
        setRadialGradientFill(image, graphics, DEFAULT_RADIAL_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_RADIAL_GRADIENT_PAINT_SECOND_COLOR, DEFAULT_RADIAL_GRADIENT_PAINT_THIRD_COLOR, cycleMethod, radiusScale);
    }

    /**
     * Set radial gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setRadialGradientFill(TestImage image, Graphics2D graphics)
    {
        setRadialGradientFill(image, graphics, DEFAULT_RADIAL_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_RADIAL_GRADIENT_PAINT_SECOND_COLOR, DEFAULT_RADIAL_GRADIENT_PAINT_THIRD_COLOR);
    }

    /**
     * Set radial gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param cycleMethod
     *            the method to use when painting outside gradient bounds
     */
    public static void setRadialGradientFill(TestImage image, Graphics2D graphics, CycleMethod cycleMethod)
    {
        setRadialGradientFill(image, graphics, DEFAULT_RADIAL_GRADIENT_PAINT_FIRST_COLOR, DEFAULT_RADIAL_GRADIENT_PAINT_SECOND_COLOR, DEFAULT_RADIAL_GRADIENT_PAINT_THIRD_COLOR, cycleMethod, 1.0f);
    }

    /**
     * Set radial gradient fill for given graphics context.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color used in the gradient
     * @param color2
     *            color used in the gradient
     * @param color3
     *            color used in the gradient
     */
    public static void setRadialGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, Color color3)
    {
        setRadialGradientFill(image, graphics, color1, color2, color3, CycleMethod.NO_CYCLE, 1.0f);
    }

    /**
     * Set radial gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param color1
     *            color used in the gradient
     * @param color2
     *            color used in the gradient
     * @param color3
     *            color used in the gradient
     * @param cycleMethod
     *            the method to use when painting outside gradient bounds
     * @param radiusScale
     *            scale used while calculating radius
     */
    public static void setRadialGradientFill(TestImage image, Graphics2D graphics, Color color1, Color color2, Color color3, CycleMethod cycleMethod, float radiusScale)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = (int) (CommonShapesRenderer.calculateRadius(image) * radiusScale);
        float[] dist = { 0.0f, 0.2f, 1.0f };
        Color[] colors = { color1, color2, color3 };
        RadialGradientPaint gradientPaint = new RadialGradientPaint(xc, yc, radius, dist, colors, cycleMethod);
        graphics.setPaint(gradientPaint);
    }

    /**
     * Set radial gradient fill for given graphics context.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @param fractions
     *            numbers ranging from 0.0 to 1.0 specifying the distribution of
     *            colors along the gradient
     * @param colors
     *            array of colors corresponding to each fractional value
     */
    public static void setRadialGradientFill(TestImage image, Graphics2D graphics, float[] fractions, Color[] colors)
    {
        // calculate center of the image
        int xc = image.getCenterX();
        int yc = image.getCenterY();

        // calculate radius of circle
        int radius = CommonShapesRenderer.calculateRadius(image);
        RadialGradientPaint gradientPaint = new RadialGradientPaint(xc, yc, radius, fractions, colors);
        graphics.setPaint(gradientPaint);
    }

    /**
     * Set texture paint using simple checker texture containing just two colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingCheckerTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getCheckerTexturePaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using simple diagonal checker texture containing just two colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingDiagonalCheckerTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getDiagonalCheckerTexturePaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using grid texture where one-pixel width lines are used.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingGridTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getGridTexturePaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using diagonal grid texture where one-pixel width lines
     * are used. Grid angle is set to 45 degrees.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingDiagonalGridTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getDiagonalGridTexturePaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using texture consisting of horizontal stripes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingHorizontalStripesTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getHorizontalStripesPaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using texture consisting of vertical stripes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingVerticalStripesTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getVerticalStripesPaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using texture consisting of horizontal color stripes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingHorizontalColorStripesTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getHorizontalColorStripesPaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using texture consisting of vertical color stripes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingVerticalColorStripesTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getVerticalColorStripesPaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using texture consisting of diagonal stripes.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingDiagonalStripesTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getDiagonalStripesPaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using texture consisting of color dots.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingColorDotsTexture(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getColorDotsPaint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using RGB texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingRGBTexture1(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getRGBTexture1Paint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using RGB texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingRGBTexture2(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getRGBTexture2Paint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using RGB texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingRGBTexture3(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getRGBTexture3Paint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using RGB texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingRGBTexture4(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getRGBTexture4Paint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using RGB texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingRGBTexture5(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getRGBTexture5Paint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set texture paint using RGB texture.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     */
    public static void setTextureFillUsingRGBTexture6(TestImage image, Graphics2D graphics)
    {
        // compute anchor for a texture
        Rectangle2D anchor = createAnchorRectangle(image);

        // create texture and paint object
        TexturePaint texturePaint = ProceduralTextureFactory.getRGBTexture6Paint(image, anchor);
        graphics.setPaint(texturePaint);
    }

    /**
     * Set zero pixels wide stroke and default cap and join style.
     * 
     * @param graphics
     *            graphics context for image
     */
    public static void setStrokeZeroThick(Graphics2D graphics)
    {
        graphics.setStroke(new BasicStroke(0));
    }

    /**
     * Set 10 pixels wide stroke and specified cap style.
     * 
     * @param graphics
     *            graphics context for image
     * @param capStyle
     *            the decoration of the ends of a BasicStroke
     */
    public static void setStrokeZeroThick(Graphics2D graphics, int capStyle)
    {
        graphics.setStroke(new BasicStroke(0, capStyle, BasicStroke.JOIN_BEVEL));
    }

    /**
     * Set 10 pixels wide stroke and specified cap style and join style.
     * 
     * @param graphics
     *            graphics context for image
     * @param capStyle
     *            the decoration of the ends of a BasicStroke
     * @param joinStyle
     *            the decoration applied where path segments meet
     */
    public static void setStrokeZeroThick(Graphics2D graphics, int capStyle, int joinStyle)
    {
        graphics.setStroke(new BasicStroke(0, capStyle, joinStyle));
    }

    /**
     * Set 10 pixels wide stroke and default cap and join style.
     * 
     * @param graphics
     *            graphics context for image
     */
    public static void setStrokeThickWidth(Graphics2D graphics)
    {
        graphics.setStroke(new BasicStroke(STROKE_WIDTH_THICK));
    }

    /**
     * Set 10 pixels wide stroke and specified cap style.
     * 
     * @param graphics
     *            graphics context for image
     * @param capStyle
     *            the decoration of the ends of a BasicStroke
     */
    public static void setStrokeThickWidth(Graphics2D graphics, int capStyle)
    {
        graphics.setStroke(new BasicStroke(STROKE_WIDTH_THICK, capStyle, BasicStroke.JOIN_BEVEL));
    }

    /**
     * Set 10 pixels wide stroke and specified cap style and join style.
     * 
     * @param graphics
     *            graphics context for image
     * @param capStyle
     *            the decoration of the ends of a BasicStroke
     * @param joinStyle
     *            the decoration applied where path segments meet
     */
    public static void setStrokeThickWidth(Graphics2D graphics, int capStyle, int joinStyle)
    {
        graphics.setStroke(new BasicStroke(STROKE_WIDTH_THICK, capStyle, joinStyle));
    }

    /**
     * Set 30 pixels wide stroke.
     *
     * @param graphics
     *            graphics context for image
     */
    public static void setStrokeExtraThickWidth(Graphics2D graphics)
    {
        graphics.setStroke(new BasicStroke(STROKE_WIDTH_EXTRA_THICK));
    }

    /**
     * Set 30 pixels wide stroke and specified cap style.
     * 
     * @param graphics
     *            graphics context for image
     * @param capStyle
     *            the decoration of the ends of a BasicStroke
     */
    public static void setStrokeExtraThickWidth(Graphics2D graphics, int capStyle)
    {
        graphics.setStroke(new BasicStroke(STROKE_WIDTH_EXTRA_THICK, capStyle, BasicStroke.JOIN_BEVEL));
    }

    /**
     * Set 30 pixels wide stroke and specified cap style and join style.
     * 
     * @param graphics
     *            graphics context for image
     * @param capStyle
     *            the decoration of the ends of a BasicStroke
     * @param joinStyle
     *            the decoration applied where path segments meet
     */
    public static void setStrokeExtraThickWidth(Graphics2D graphics, int capStyle, int joinStyle)
    {
        graphics.setStroke(new BasicStroke(STROKE_WIDTH_EXTRA_THICK, capStyle, joinStyle));
    }

    /**
     * Set default dash pattern.
     * 
     * @param graphics
     *            graphics context for image
     */
    public static void setDashedStrokePattern(Graphics2D graphics2d)
    {
        setDashedStrokePattern(graphics2d, DEFAULT_DASH_PATTERN);
    }

    /**
     * Set dash pattern for extra thick paths.
     * 
     * @param graphics
     *            graphics context for image
     */
    public static void setDashedStrokePatternForExtraThickPaths(Graphics2D graphics2d)
    {
        setDashedStrokePattern(graphics2d, DEFAULT_DASH_PATTERN_FOR_EXTRA_THICK_PATHS);
    }

    /**
     * Set dash pattern.
     * 
     * @param graphics
     *            graphics context for image
     * @param dash
     *            the array representing the dashing pattern
     */
    public static void setDashedStrokePattern(Graphics2D graphics2d, float[] dash)
    {
        BasicStroke oldStroke = (BasicStroke) graphics2d.getStroke();
        float lineWidth = oldStroke.getLineWidth();
        int endCap = oldStroke.getEndCap();
        int lineJoin = oldStroke.getLineJoin();
        float miterLimit = oldStroke.getMiterLimit();
        float dashPhase = oldStroke.getDashPhase();
        BasicStroke newStroke = new BasicStroke(lineWidth, endCap, lineJoin, miterLimit, dash, dashPhase);
        graphics2d.setStroke(newStroke);
    }

    /**
     * Create anchor rectangle used by texture paint
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @return anchor represented by {@link Rectangle2D} object
     */
    private static Rectangle2D createAnchorRectangle(TestImage image)
    {
        Rectangle2D anchor = new Rectangle2D.Double(0, 0, image.getWidth(), image.getHeight());
        return anchor;
    }

}
