/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012, 2013  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Map;

import org.gfxtest.common.Configuration;
import org.gfxtest.common.ConfigurationException;
import org.gfxtest.common.InvalidParameterValueException;



/**
 * Configuration set by gfx-test harness for each test suite. Configuration
 * parameters could be specified as command line arguments.
 * 
 * @author Pavel Tisnovsky
 */
public class GfxTestConfiguration extends Configuration
{
    /**
     * Default width of the test image (measured in pixels).
     * This value is used in case the "-w" parameter is not used.
     */
    public static final int DEFAULT_IMAGE_WIDTH = 640;

    /**
     * Default height of the test image (measured in pixels).
     * This value is used in case the "-h" parameter is not used.
     */
    public static final int DEFAULT_IMAGE_HEIGHT = 480;

    /**
     * Default image type.
     * This value is used in case the "-t" parameter is not used.
     */
    public static final String DEFAULT_IMAGE_TYPE = "rgb";

    /**
     * Default output directory.
     * This value is used in case the "-o" parameter is not used.
     */
    private static final String DEFAULT_OUTPUT_DIRECTORY = ".";

    /**
     * Path to generated output files.
     */
    private File outputPath;

    /**
     * Width (measured in pixels) of the generated images.
     */
    private int imageWidth;

    /**
     * Height (measured in pixels) of the generated images.
     */
    private int imageHeight;

    /**
     * Image type. Currently are supported these image types:
     * BufferedImage.TYPE_INT_RGB, BufferedImage.TYPE_BYTE_GRAY and
     * BufferedImage.TYPE_BYTE_INDEXED.
     */
    private int imageType;

    /**
     * Filter, ie. part of test name. Only tests which contains this pattern
     * in its names will be run.
     */
    private String filter;

    /**
     * Constructor called indirectly when the test suite are started.
     * 
     * @param args
     *            command line arguments
     * @param log
     *            logger object
     * @throws ConfigurationException
     */
    public GfxTestConfiguration(String[] args, Log log) throws ConfigurationException
    {
        super(args, log);
    }

    /**
     * This method reads and process all command line parameters.
     * 
     * @param args
     *            command line arguments
     */
    @Override
    protected void readCommandLineParameters(String[] args) throws ConfigurationException
    {
        Map<String, String> options = resolveAllOptions(args);
        this.imageWidth = processImageWidthArgument(options);
        this.imageHeight = processImageHeightArgument(options);
        this.imageType = processImageTypeArgument(options);
        this.outputPath = processOutputPathArgument(options);
        this.filter = processFilter(options);
    }

    /**
     * Process argument used for setting image width.
     * 
     * @param options
     *            command line options
     * @return image width
     * @throws ConfigurationException
     */
    private int processImageWidthArgument(Map<String, String> options) throws ConfigurationException
    {
        try
        {
            return getIntegerParameter(options, "w"); //$NON-NLS-1$
        }
        catch (ParameterNotFoundException e)
        {
            this.log.logWarning("image width not specified in command line, using default setting: "
                            + DEFAULT_IMAGE_WIDTH);
            return DEFAULT_IMAGE_WIDTH;
        }
    }

    /**
     * Process argument used for setting image height.
     * 
     * @param options
     *            command line options
     * @return image height
     * @throws ConfigurationException
     */
    private int processImageHeightArgument(Map<String, String> options) throws ConfigurationException
    {
        try
        {
            return getIntegerParameter(options, "h"); //$NON-NLS-1$
        }
        catch (ParameterNotFoundException e)
        {
            this.log.logWarning("image height not specified in command line, using default setting: "
                            + DEFAULT_IMAGE_HEIGHT);
            return DEFAULT_IMAGE_HEIGHT;
        }
    }

    /**
     * Process argument used for setting image type (RGB, GrayScale, etc.)
     * 
     * @param options
     *            command line options
     * @return image type
     * @throws ConfigurationException
     */
    private int processImageTypeArgument(Map<String, String> options) throws ConfigurationException
    {
        try
        {
            return getImageType(options);
        }
        catch (ParameterNotFoundException e)
        {
            this.log.logWarning("image type not specified in command line, using default setting: "
                            + BufferedImage.TYPE_INT_RGB);
            return BufferedImage.TYPE_INT_RGB;
        }
    }

    /**
     * Process argument used for setting output path (the name of directory
     * where test images are stored).
     * 
     * @param options
     *            command line options
     * @return output path
     * @throws ConfigurationException
     */
    private File processOutputPathArgument(Map<String, String> options) throws ConfigurationException
    {
        try
        {
            return new File(getStringParameter(options, "o")); //$NON-NLS-1$;
        }
        catch (ParameterNotFoundException e)
        {
            this.log.logWarning("output path not specified in command line, using default setting: "
                            + DEFAULT_OUTPUT_DIRECTORY);
            return new File(DEFAULT_OUTPUT_DIRECTORY);
        }
    }

    /**
     * Process argument used for setting test filter.
     *
     * @param options
     *            command line options
     * @return test filter
     * @throws ConfigurationException when configuration is wrong
     */
    private String processFilter(Map<String, String> options) throws ConfigurationException
    {
        try
        {
            return getStringParameter(options, "f"); //$NON-NLS-1$
        }
        catch (ParameterNotFoundException e)
        {
            this.log.logWarning("filter not specified in command line, using default setting: null");
            return null;
        }
    }

    /**
     * This method prints all test suite parameters.
     */
    @Override
    @SuppressWarnings("nls")
    protected void printParameters()
    {
        this.log.logSet("output path", this.outputPath);
        this.log.logSet("image width", Integer.valueOf(this.imageWidth));
        this.log.logSet("image height", Integer.valueOf(this.imageHeight));
        this.log.logSet("image type", Integer.valueOf(this.imageType));
        this.log.logSet("filter", this.filter);
    }

    /**
     * Process image type parameter.
     * 
     * @param options
     *            command line options
     * @return image type
     * @throws ConfigurationException
     */
    @SuppressWarnings("nls")
    private int getImageType(Map<String, String> options) throws ConfigurationException
    {
        if (!options.containsKey("t"))
        {
            throw new ParameterNotFoundException("t");
        }
        String imgType = options.get("t");
        if ("rgb".equals(imgType))
        {
            return BufferedImage.TYPE_INT_RGB;
        }
        else if ("gray".equals(imgType))
        {
            return BufferedImage.TYPE_BYTE_GRAY;
        }
        else if ("bw".equals(imgType) || ("bit".equals(imgType)))
        {
            return BufferedImage.TYPE_BYTE_INDEXED;
        }
        else
        {
            throw new InvalidParameterValueException("t", imgType);
        }
    }

    /**
     * Getter for outputPath attribute.
     * 
     * @return current value of attribute outputPath.
     */
    public File getOutputPath()
    {
        return this.outputPath;
    }

    /**
     * Getter for imageWidth attribute.
     * 
     * @return current value of attribute imageWidth.
     */
    public int getImageWidth()
    {
        return this.imageWidth;
    }

    /**
     * Getter for imageHeight attribute.
     * 
     * @return current value of attribute imageHeight.
     */
    public int getImageHeight()
    {
        return this.imageHeight;
    }

    /**
     * Getter for imageType attribute.
     * 
     * @return current value of attribute imageType.
     */
    public int getImageType()
    {
        return this.imageType;
    }

    /**
     * Getter for filter attribute.
     * 
     * @return current value of attribute filter.
     */
    public String getFilter()
    {
        return this.filter;
    }

}
